/*
  
  This file is part of ERIKYYYPHONE
  Copyright (C) 1996 Erik Thiele
  see README for details and contact adresses.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  */

#include "MSG.H"
#include "fatal.H"
#include "warter.H"
#include "msgarten.H"
#include "erikyyylist.H"
#include "CC_oberfl.H"
#include "Ccommon.H"
#include "version.H"

#include <signal.h>
#include <netinet/in.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>

MSG_client *cl;
MSG_communicator *com;


void Ccommon_output(uint8 msgart,char *message,uint16 len)
{
  com->sendmsg(msgart,message,len);
};


int main(int argc, char **argv)
{
  int32 serverhostip;

  printf("\n*** ERIKYYYPHONE "erikyyyphoneversion" CLIENT ***\n\n");

  // WARTER init
  WARTER_init();

  // Terminiersignale
  WARTER_add(WARTER_SIGNAL,SIGHUP);
  WARTER_add(WARTER_SIGNAL,SIGINT);
  WARTER_add(WARTER_SIGNAL,SIGQUIT);
  WARTER_add(WARTER_SIGNAL,SIGABRT);
  WARTER_add(WARTER_SIGNAL,SIGTERM);
  WARTER_add(WARTER_SIGNAL,SIGPWR);

  // Zu ignoierende Signale
  WARTER_add(WARTER_SIGNAL,SIGIO);
  WARTER_add(WARTER_SIGNAL,SIGPIPE);
  WARTER_add(WARTER_SIGNAL,SIGTSTP);
  WARTER_add(WARTER_SIGNAL,SIGTTIN);
  WARTER_add(WARTER_SIGNAL,SIGTTOU);

  {
    printf("Parsing options...\n");
    int opt,optionerror=0,haveip=0;
    opterr=0;
    while ((opt = getopt(argc, argv, "h:")) != EOF) {
      switch (opt)
	{
	case 'h':
	  if (haveip) {
	    printf("*** You specified more that one \"-h\" option\n");
	    optionerror=1;
	  }else{
	    if (MSG_string2ip(optarg,&serverhostip)) {
	      char *ips;
	      MSG_ip2string(&ips,serverhostip);
	      printf("I will connect the server at %s\n",ips);
	      free(ips);
	      haveip=1;
	    }else{
	      printf("*** You specified an invalid IP in a \"-h\" option\n");
	      optionerror=1;
	    };
	  };
	  break;
	case ':':
	  printf("*** There is a missing parameter for one of the options\n");
	  optionerror=1;
	  break;
	case '?':
	  printf("*** You specified an unknown option, or forgot a parameter to an option\n");
	  optionerror=1;
	  break;
	default:
	  optionerror=1;
	};
    };
    if (optind<argc) {
      printf("*** You added some unknown characters after your options\n");
      optionerror=1;
    };
    if (!haveip) {
      printf("*** You forgot to specify the \"-h\" option\n");
      optionerror=1;
    };
    if (optionerror) {
      printf("You made some mistake specifying the options...\n"
	     "\n"
	     "\n"
	     "erikyyyphone -h IP\n"
	     "\n"
	     "The -h option specifies the IP of the server you want to connect to.\n"
	     "\n"
	     "EXAMPLE:\n"
	     "\n"
	     "erikyyyphone -h 127.0.0.1\n"
	     "\n"
	     "an erikyyyphoneserver must be running at 127.0.0.1, otherwise this\n"
	     "won't work\n");
      exit(1);
    };
    printf("...option parsing done\n");
  };

  // MSG_Client initialisieren
  cl=new MSG_client(serverhostip,htons(2900));

  // MSG_communicator initialisieren
  com=0;

  // Ccommon initialisieren
  Ccommon_init();

  // OBERFLAECHE init
  OBERFL_init();
  Ccommon_logmsg("Connecting the server...");

  int ende=0;
  do{
    // ***** OBERFL_update
    OBERFL_update();

    // ***** Event Scheduler
    WARTER_warte();

    // ***** Verwalte den MSG_client
    if (cl) {
      int neu;
      MSG_client::statustype erg=cl->connectready(&neu);
      if (erg==MSG_client::ERROR) fatalfehler("An error occured trying to connect to the Server");
      if (erg==MSG_client::CONNECTED) {
	Ccommon_logmsg("Server connected. Identifying...");
	delete cl;
	cl=0;
	com=new MSG_communicator(neu);
      };
    };

    // ***** Communiziere mit dem Server
    if (com) {
      if (!(com->handleevent())) ende=1;

      if (com->msgavail()) {
	char *message;
	uint8 msgart;
	uint16 len;
	com->recvmsg(&msgart,&message,&len);
	Ccommon_input(msgart,message,len);
	free(message);
      };
    };

    // ***** Reagiere auf Signale
    if (WARTER_signal_aktiv_und_reset(SIGINT)||WARTER_signal_aktiv_und_reset(SIGTERM)||
	WARTER_signal_aktiv_und_reset(SIGHUP)||WARTER_signal_aktiv_und_reset(SIGQUIT)||
	WARTER_signal_aktiv_und_reset(SIGPWR)||WARTER_signal_aktiv_und_reset(SIGABRT)) ende=1;
  }while(!ende);

  // Oberflaeche deinit
  OBERFL_deinit();

  // Ccommon deinitialisieren
  Ccommon_deinit();

  // MSG_communicator deinitialisieren
  if (com) delete com;

  // MSG_Client deinitialisieren
  if (cl) delete cl;

  // Terminiersignale
  WARTER_del(WARTER_SIGNAL,SIGHUP);
  WARTER_del(WARTER_SIGNAL,SIGINT);
  WARTER_del(WARTER_SIGNAL,SIGQUIT);
  WARTER_del(WARTER_SIGNAL,SIGABRT);
  WARTER_del(WARTER_SIGNAL,SIGTERM);
  WARTER_del(WARTER_SIGNAL,SIGPWR);

  // Zu ignoierende Signale
  WARTER_del(WARTER_SIGNAL,SIGIO);
  WARTER_del(WARTER_SIGNAL,SIGPIPE);
  WARTER_del(WARTER_SIGNAL,SIGTSTP);
  WARTER_del(WARTER_SIGNAL,SIGTTIN);
  WARTER_del(WARTER_SIGNAL,SIGTTOU);

  // WARTER deinit
  WARTER_deinit();

  printf("The connection to the server is closed.\n\n");

  return(0);
};
