/*
  
  This file is part of ERIKYYYPHONE
  Copyright (C) 1996 Erik Thiele
  see README for details and contact adresses.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  */

#include "CC_oberfl.H"

#include "MSG.H"
#include "erikyyylist.H"
#include "GUI.H"
#include "Ccommon.H"
#include "fatal.H"

#include <stdlib.h>
#include <stdio.h>


static char *editstring;
static int recordstat;
static int recordwishstat;
static int spalte;


// *************************** Ccommon_recordwishstat

void Ccommon_recordwishstat(int anaus)
{
  recordwishstat=anaus;
};


// *************************** Ccommon_recordstat

void Ccommon_recordstat(int anaus)
{
  recordstat=anaus;
};


// *************************** servcondisp

class servcondisp {
public:
  char *ipstr;
  serverconnection_statustype st;
  
  servcondisp(serverconnection_statustype *_st)
  {
    st=*_st;
    MSG_ip2string(&ipstr,st.ip);
  };

  ~servcondisp()
  {
    free(ipstr);
  };
};

static liste<servcondisp> *servcondispli;
static liste_elem<servcondisp> *cursorservernode;


// *************************** outcondisp

class outcondisp {
public:
  char *ipstr;
  outconnection_statustype st;

  outcondisp(outconnection_statustype *_st)
  {
    st=*_st;
    MSG_ip2string(&ipstr,st.ip);
  };

  ~outcondisp()
  {
    free(ipstr);
  };
};

static liste<outcondisp> *outcondispli;
static liste_elem<outcondisp> *cursoroutnode;


// *************************** Ccommon_serverconnect_addnew

void Ccommon_serverconnect_addnew(serverconnection_statustype *st)
{
  servcondispli->addfront(new servcondisp(st));
};


// *************************** Ccommon_serverconnect_del

void Ccommon_serverconnect_del(int32 ip)
{
  liste_elem<servcondisp> *akt=servcondispli->first;
  while(akt) {
    if (akt->objptr->st.ip==ip) {
      if (cursorservernode==akt)cursorservernode=NULL;
      servcondispli->del(akt);
      return;
    };
    akt=akt->next;
  };
  fatalfehler("Ccommon_serverconnect_del: serverconnection not found");
};


// *************************** Ccommon_serverconnect_modify

void Ccommon_serverconnect_modify(serverconnection_statustype *st)
{
  liste_elem<servcondisp> *akt=servcondispli->first;
  while(akt) {
    if (akt->objptr->st.ip==st->ip) {
      akt->objptr->st=*st;
      return;
    };
    akt=akt->next;
  };
  fatalfehler("CC_oberfl.C: Ccommon_serverconnect_modify: serverconnection not found");
};


// *************************** Ccommon_outconnect_addnew

void Ccommon_outconnect_addnew(outconnection_statustype *st)
{
  outcondispli->addfront(new outcondisp(st));
};


// *************************** Ccommon_outconnect_del

void Ccommon_outconnect_del(int32 ip)
{
  liste_elem<outcondisp> *akt=outcondispli->first;
  while(akt) {
    if (akt->objptr->st.ip==ip) {
      if (cursoroutnode==akt)cursoroutnode=NULL;
      outcondispli->del(akt);
      return;
    };
    akt=akt->next;
  };
  fatalfehler("Ccommon_outconnect_del: outconnection not found");
};


// *************************** Ccommon_outconnect_modify

void Ccommon_outconnect_modify(outconnection_statustype *st)
{
  liste_elem<outcondisp> *akt=outcondispli->first;
  while(akt) {
    if (akt->objptr->st.ip==st->ip) {
      akt->objptr->st=*st;
      return;
    };
    akt=akt->next;
  };
  fatalfehler("CC_oberfl.C: Ccommon_outconnect_modify: outconnection not found");
};


// *************************** logger

class logelem {
public:
  char *str;
  logelem(char *w)
  {
    str=(char*)malloc(strlen(w)+1);
    strcpy(str,w);
  };
  ~logelem()
  {
    free(str);
  };
};

liste<logelem> *loglist;

void Ccommon_logmsg(char *msg)
{
  loglist->addfront(new logelem(msg));
};


// *************************** OBERFL_init

void OBERFL_init()
{
  GUI_init(80,23);
  GUI_cursor_visibility(1);
  servcondispli=new liste<servcondisp>();
  outcondispli=new liste<outcondisp>();
  loglist=new liste<logelem>();
  editstring=(char*)malloc(50);
  *editstring=0;
  recordstat=0;
  recordwishstat=0;
  spalte=0;
  cursorservernode=NULL;
  cursoroutnode=NULL;
};


// *************************** OBERFL_deinit
void OBERFL_deinit()
{
  free(editstring);
  delete loglist;
  delete servcondispli;
  delete outcondispli;
  GUI_deinit();
};


// *************************** OBERFL_update

void OBERFL_update()
{
  // Parse gedrueckte keys
  {
    int k;
    if (GUI_getch(&k)) {
      switch (k)
	{
	case '+':
	  if ((!spalte)&&cursorservernode) {
	    MSG_SERVERCONNECTION_SET_struct s;
	    s.ip=cursorservernode->objptr->st.ip;
	    s.rellatency=8;
	    Ccommon_output (MSG_SERVERCONNECTION_SET,(char*)&(s),sizeof(s));
	  };
	  break;
	case '-':
	  if ((!spalte)&&cursorservernode) {
	    MSG_SERVERCONNECTION_SET_struct s;
	    s.ip=cursorservernode->objptr->st.ip;
	    s.rellatency=-8;
	    Ccommon_output (MSG_SERVERCONNECTION_SET,(char*)&(s),sizeof(s));
	  };
	  break;
	case ' ':
	  if (Ccommon_ready) {
	    Ccommon_output (MSG_PLEASE_TOGGLE_RECORDING, NULL, 0);
	  }else{
	    Ccommon_logmsg ("Server isn't ready yet.");
	  };
	  break;
	case 'k':
	  if (spalte) {
	    if (cursoroutnode) {
	      Ccommon_output (MSG_PLEASE_KILL_OUTCONNECTION,(char*)&(cursoroutnode->objptr->st.ip),
			      sizeof(cursoroutnode->objptr->st.ip));
	    };
	  }else{
	    if (cursorservernode) {
	      Ccommon_output (MSG_PLEASE_KILL_SERVERCONNECTION,(char*)&(cursorservernode->objptr->st.ip),
			      sizeof(cursorservernode->objptr->st.ip));
	    };
	  };
	  break;
	case 13:
	  if (Ccommon_ready) {
	    int32 ip;
	    if (MSG_string2ip(editstring,&ip)) {
	      Ccommon_output (MSG_PLEASE_CONNECT_A_NEW_SERVER, (char*)&ip, sizeof(ip));
	    }else{
	      Ccommon_logmsg ("not a valid IP");
	    };
	    *editstring=0;
	  }else{
	    Ccommon_logmsg ("Server isn't ready yet.");
	  };
	  break;
	default:
	  {
	    if ((k=='0')||(k=='1')||(k=='2')||(k=='3')||(k=='4')||(k=='5')||
		(k=='6')||(k=='7')||(k=='8')||(k=='9')||(k=='.')) {
	      int a=strlen(editstring);
	      if (a<3*4+3) {
		editstring[a]=k;
		editstring[a+1]=0;
	      };
	    } else if (k==GUI_KEYDELETE) {
	      int a=strlen(editstring);
	      if (a) {
		editstring[a-1]=0;
	      };
	    } else if (k==GUI_KEYUP) {
	      if (spalte) {
		if (cursoroutnode) {
		  if (cursoroutnode->prev)
		    cursoroutnode=cursoroutnode->prev;
		}else{
		  cursoroutnode=outcondispli->first;
		};
	      }else{
		if (cursorservernode) {
		  if (cursorservernode->prev)
		    cursorservernode=cursorservernode->prev;
		}else{
		  cursorservernode=servcondispli->first;
		};
	      };
	    } else if (k==GUI_KEYDOWN) {
	      if (spalte) {
		if (cursoroutnode) {
		  if (cursoroutnode->next) cursoroutnode=cursoroutnode->next;
		}else{
		  cursoroutnode=outcondispli->first;
		};
	      }else{
		if (cursorservernode) {
		  if (cursorservernode->next) cursorservernode=cursorservernode->next;
		}else{
		  cursorservernode=servcondispli->first;
		};
	      };
	    } else if (k==GUI_KEYLEFT) {
	      if (spalte)spalte--;
	    } else if (k==GUI_KEYRIGHT) {
	      if (spalte<1)spalte++;
	    };	    
	    break;
	  };
	};
    };
  };
  
  int x,y;
  
  // General Layout
  GUI_putbox(0,0,GUI_getanzx()-1,GUI_getanzy()-1,' ');  // General CLS
  
  GUI_putrahmen(0,0,GUI_getanzx()-1,GUI_getanzy()-1);
  GUI_putcenterstring(GUI_getanzx()>>1,1,"ERIKYYYPHONE");
  GUI_puthline(6,2,GUI_getanzx()-7);
  GUI_putvline(GUI_getanzx()>>1,3,GUI_getanzy()-11);
  GUI_puthline(1,GUI_getanzy()-8,GUI_getanzx()-2);
  GUI_puthline(1,GUI_getanzy()-10,GUI_getanzx()-2);
  GUI_putstring(4,GUI_getanzy()-9,recordstat ? " ** RECORD ** " : "    RECORD    " );
  GUI_putstring(20,GUI_getanzy()-9,recordwishstat ? " ** RECORDWISH ** " : "    RECORDWISH    " );
  GUI_putstring(40,GUI_getanzy()-9,editstring);
  GUI_cursor_setpos(40+strlen(editstring),GUI_getanzy()-9);
  
  // incoming connects
  {
    GUI_putstring(5,4,"incoming connects");
    
    x=3;
    y=6;
    liste_elem<servcondisp> *akt=servcondispli->first;
    while ((y!=GUI_getanzy()-11)&&(akt)) {
      if ((spalte==0)&&(cursorservernode==akt)) {
	GUI_reverse(1);
	GUI_putbox(x-1,y,x+(GUI_getanzx()>>1)-5,y,' ');
      };
      GUI_putstring(x,y,akt->objptr->ipstr);
      
      switch (akt->objptr->st.status)
	{
	case serverconnection_statustype::ON_AIR:GUI_putstring(x+16,y,"PLAY");break;
	case serverconnection_statustype::IDLE:GUI_putstring(x+16,y,"IDLE");break;
	case serverconnection_statustype::BUFFER:GUI_putstring(x+16,y,"BUFF");break;
	case serverconnection_statustype::WANT_TO_PLAY:GUI_putstring(x+16,y,"WANT");break;
	};
      
      int breit = (akt->objptr->st.buffill)>>3;
      if (breit > (GUI_getanzx()>>1)-29) breit=(GUI_getanzx()>>1)-29;
      for (int a=0;a<breit;a++) GUI_putchar(x+24+a,y,'=');
      if (akt->objptr->st.latency>=0) {
	GUI_putchar(x+24+((akt->objptr->st.latency)>>3),y,'|');
      }else{
	GUI_putchar(x+24,y,'N');
      };
      char msg[50];
      sprintf(msg,"%d",akt->objptr->st.buffill);
      GUI_putstring(x+21,y,msg);
      
      y++;
      akt=akt->next;
      GUI_reverse(0);
    };
  };
  
  // outgoing connects
  {
    GUI_putstring((GUI_getanzx()>>1)+5,4,"outgoing connects");
    
    x=(GUI_getanzx()>>1)+3;
    y=6;
    liste_elem<outcondisp> *akt=outcondispli->first;
    while ((y!=GUI_getanzy()-11)&&(akt)) {
      if ((spalte==1)&&(cursoroutnode==akt)) {
	GUI_reverse(1);
	GUI_putbox(x-1,y,x+((GUI_getanzx()-1)>>1)-5,y,' '); 
      };
      GUI_putstring(x,y,akt->objptr->ipstr);
      
      switch (akt->objptr->st.status)
	{
	case outconnection_statustype::CONNECTING:GUI_putstring(x+16,y,"CONN");break;
	case outconnection_statustype::ON_AIR:GUI_putstring(x+16,y,"PLAY");break;
	case outconnection_statustype::IDLE:GUI_putstring(x+16,y,"IDLE");break;
	case outconnection_statustype::BUFFER:GUI_putstring(x+16,y,"BUFF");break;
	case outconnection_statustype::WANT_TO_PLAY:GUI_putstring(x+16,y,"WANT");break;
	};
      
      int breit = (akt->objptr->st.buffill)>>3;
      if (breit > ((GUI_getanzx()-1)>>1)-29) breit=((GUI_getanzx()-1)>>1)-29;
      for (int a=0;a<breit;a++) GUI_putchar(x+24+a,y,'=');
      
      char msg[50];
      sprintf(msg,"%d",akt->objptr->st.buffill);
      GUI_putstring(x+21,y,msg);
      
      y++;
      akt=akt->next;
      GUI_reverse(0);
    };
  };
  
  // showing the message log
  {
    liste_elem<logelem> *akt=loglist->first;
    x=3;
    y=GUI_getanzy()-3;
    while(akt) {
      if (y<GUI_getanzy()-6) {
	loglist->del_and_next(&akt);
      }else{
	GUI_putstring(x,y,akt->objptr->str);
	y--;
	akt=akt->next;
      };
    };
  };
  
  /*      // Usage counter...
	{
	static char s[100];
	static int r=0;
	sprintf(s,"%d",r++);
	GUI_putstring(0,0,s);
	};*/
  
  // updating the screen
  GUI_update();
};
