/*
  
  This file is part of ERIKYYYPHONE
  Copyright (C) 1996 Erik Thiele
  see README for details and contact adresses.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  */

#define MSG_NEED_SPECIAL_UTILITIES
#include "MSG.H"

#include "warter.H"
#include "fatal.H"
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <stdlib.h>
#include <stdio.h>
#include <arpa/inet.h>
#include <errno.h>


/*******************************************************************
  
                         common utilities

*******************************************************************/

void set_socket_reuse(int handle)
{
  int reuser=1;
  if (setsockopt(handle,SOL_SOCKET,SO_REUSEADDR,&reuser,sizeof(reuser))<0)
    fatalfehlererrno("set_socket_reuse: setsockopt SO_REUSEADDR failed");
};

void set_socket_linger(int handle)
{
  struct linger ling;
  memset(&ling,0,sizeof(ling));
  if (setsockopt(handle,SOL_SOCKET,SO_LINGER,&ling,sizeof(ling))<0)
    fatalfehlererrno("set_socket_linger: setsockopt SO_LINGER failed");
};

void set_socket_nonblock(int handle)
{
  if (fcntl(handle,F_SETFL,O_NONBLOCK)<0)
    fatalfehlererrno("set_socket_nonblock: fcntl O_NONBLOCK failed");
};

void create_socket(int *handle)
{
  *handle = socket(AF_INET,SOCK_STREAM,0);
  if ((*handle)<0) fatalfehlererrno("create_socket: socket(AF_INET,SOCK_STREAM,0)<0");
};

/*void create_UDP_socket(int *handle)
{
  *handle = socket(AF_INET,SOCK_DGRAM,0);
  if ((*handle)<0) fatalfehlererrno("create_UDP_socket: socket(AF_INET,SOCK_DGRAM,0)<0");
};*/

void bind_socket(int handle,uint16 portnum)
{
  sockaddr_in from;
  memset(&from,0,sizeof(from));
  from.sin_family=AF_INET;
  from.sin_addr.s_addr=htonl(INADDR_ANY);
  from.sin_port=portnum;

  if (bind(handle,(sockaddr *)&from,sizeof(from))<0) 
    fatalfehlererrno("bind_socket: bind(...)<0");
};

void listen_socket(int handle)
{
  if (listen(handle,5)<0)
    fatalfehlererrno("listen_socket: listen(int,5)<0");
};

void accept_socket(int handle,int *neuhandle,int32 *herkunftip)
{
  sockaddr_in from;
  memset(&from,0,sizeof(from));
  socklen_t len=sizeof(from);
  
  if ((*neuhandle=accept(handle,(sockaddr*)&from,&len))<0)
    fatalfehlererrno("accept_socket: accept(...)<0");
  
  *herkunftip=from.sin_addr.s_addr;
};

int connect_socket(int handle,int32 zielip,uint16 portnum)
{
  sockaddr_in outgoing;
  memset(&outgoing,0,sizeof(outgoing));
  *((int32*)(&outgoing.sin_addr))=zielip;
  outgoing.sin_family = AF_INET;
  outgoing.sin_port = portnum;
  
  return(connect(handle, (sockaddr*)(&outgoing), sizeof(outgoing)));
};

void MSG_ip2string(char **str,int32 ip)
{
  *str=(char*)malloc(3*4+3+1 + 10);
  sprintf(*str,"%d.%d.%d.%d",(ip      ) & 0xff,(ip >>  8) & 0xff,(ip >> 16) & 0xff,(ip >> 24) & 0xff);
  *str=(char*)realloc(*str,strlen(*str)+1);
};

int MSG_string2ip(char *str,int32 *ip)
{
  struct in_addr in;
  if (!(inet_aton(str,&in))) return(0);
  *ip=*((int32*)(&in));
  return(1);
};


/*******************************************************************
  
                             MSG_server

*******************************************************************/

MSG_server::MSG_server(uint16 portnum)
{
  create_socket(&handle);
  set_socket_reuse(handle);
  set_socket_linger(handle);
  set_socket_nonblock(handle);
  bind_socket(handle,portnum);
  listen_socket(handle);
  WARTER_add(WARTER_READ,handle);
};

MSG_server::~MSG_server()
{
  WARTER_del(WARTER_READ,handle);
  close(handle);
};

int MSG_server::acceptnew(int *neuhandle,int32 *herkunftip)
{
  if (WARTER_handle_aktiv(WARTER_READ,handle)) {
    accept_socket(handle,neuhandle,herkunftip);
    set_socket_reuse(*neuhandle);
    set_socket_linger(*neuhandle);
    WARTER_reenter=1;
    return(1);
  }else{
    return(0);
  };
};


/*******************************************************************
  
                             MSG_client

*******************************************************************/

MSG_client::MSG_client(int32 zielip_,uint16 portnum_)
{
  zielip=zielip_;
  portnum=portnum_;

  create_socket(&handle);
  set_socket_reuse(handle);
  set_socket_linger(handle);
  set_socket_nonblock(handle);

  if (connect_socket(handle,zielip,portnum)<0) {
    if (errno==EINPROGRESS) {
      status=CONNECTING;
      WARTER_add(WARTER_WRITE,handle);
    }else{
      status=ERROR;
    };
  }else{
    status=CONNECTED;
  };
};

MSG_client::~MSG_client()
{
  if (status==CONNECTING) WARTER_del(WARTER_WRITE,handle);
  if (handle) close(handle);
};

MSG_client::statustype MSG_client::connectready(int *neuhandle)
{
  if (status==CONNECTING) {
    if (WARTER_handle_aktiv(WARTER_WRITE,handle)) {
      status = (connect_socket(handle,zielip,portnum)<0) ? ERROR : CONNECTED;
      WARTER_del(WARTER_WRITE,handle);
    }else{
      *neuhandle=-1;
    };
  };

  if (status==ERROR) *neuhandle=-1;

  if (status==CONNECTED) {
    if (!handle)
      fatalfehler("MSG_client::connectready wurde aufgerufen, obwohl frueher schon connected war.");
    *neuhandle=handle;
    handle=0;
  };

  return(status);
};


/*******************************************************************
  
                          MSG_communicator

*******************************************************************/

#define READBLOCKSIZE (16384)
#define COMMUNICATOR_MAXMESSAGESIZE (16384)

MSG_communicator::MSG_communicator(int _handle)
{
  outbuffer=new buffer();
  inbuffer=new buffer();
  handle=_handle;

  set_socket_nonblock(handle);

  write_wait_stat=0;
  WARTER_add(WARTER_READ,handle);
};

MSG_communicator::~MSG_communicator()
{
  WARTER_del(WARTER_READ,handle);
  write_wait(0);
  close (handle);

  delete outbuffer;
  delete inbuffer;
};

void MSG_communicator::write_wait(int anaus)
{
  if ((anaus)&&(!write_wait_stat)) {
    WARTER_add(WARTER_WRITE,handle);
    write_wait_stat=1;
  }else if ((!anaus)&&(write_wait_stat)) {
    WARTER_del(WARTER_WRITE,handle);
    write_wait_stat=0;
  };
};

int MSG_communicator::msgavail()
{
  uint16 len;
  if ((unsigned int)(inbuffer->bufsize)>=(sizeof(uint16))) {
    len=*((uint16*)(inbuffer->data));
    if (len>COMMUNICATOR_MAXMESSAGESIZE)
      fatalfehler("MSG_communicator::msgavail(): Message size exceeded !!!");
    if ((unsigned int)(inbuffer->bufsize)>=(sizeof(uint8))+(sizeof(uint16))+((unsigned int)(len))) {
      return(1);
    };
  };
  return(0);
};

void MSG_communicator::recvmsg(uint8 *msgart,char **buf,uint16 *len)
{
  if (!msgavail()) fatalfehler("MSG_communicator::recvmsg: aufgerufen, obwohl keine msg verfuegbar.");
  *len=*((uint16*)(inbuffer->data));
  *msgart=*((uint8*)((inbuffer->data)+(sizeof(uint16))));
  *buf=(*len) ? ((char*)malloc(*len)) : ((char*)NULL);
  memcpy(*buf,(inbuffer->data)+(sizeof(uint8))+(sizeof(uint16)),*len);
  inbuffer->vornekill((sizeof(uint8))+(sizeof(uint16))+(*len));
  if (msgavail()) WARTER_reenter=1;
};

void MSG_communicator::sendmsg(uint8 msgart,char *buf,uint16 len)
{
  write_wait(1);

  outbuffer->changesize(   (outbuffer->bufsize)+len+(sizeof(uint8))+(sizeof(uint16))                       );
  memcpy((outbuffer->data)+(outbuffer->bufsize)-len-(sizeof(uint8))-(sizeof(uint16)),&len   ,sizeof(uint16));
  memcpy((outbuffer->data)+(outbuffer->bufsize)-len-(sizeof(uint8))                 ,&msgart,sizeof(uint8) );
  memcpy((outbuffer->data)+(outbuffer->bufsize)-len                                 ,buf    ,len           );
};

int MSG_communicator::handleevent()
{
  int erg;
  if (outbuffer->bufsize) {
    if (WARTER_handle_aktiv(WARTER_WRITE,handle)) {
      erg=write(handle,outbuffer->data,outbuffer->bufsize);
      if (erg<=0)return(0);
      outbuffer->vornekill(erg);
    };
  };
  if (!(outbuffer->bufsize))
    write_wait(0);
  
  if (WARTER_handle_aktiv(WARTER_READ,handle)) {
    inbuffer->changesize((inbuffer->bufsize)+READBLOCKSIZE);
    erg=read(handle,(inbuffer->data)+(inbuffer->bufsize)-READBLOCKSIZE,READBLOCKSIZE);
    if (erg<=0) {
      inbuffer->changesize((inbuffer->bufsize)-READBLOCKSIZE);
      return(0);
    }else{
      inbuffer->changesize((inbuffer->bufsize)-READBLOCKSIZE+erg);
    };
  };
  return(1);
};


/*******************************************************************
  
                           MSG_UDP_server

*******************************************************************/

/*MSG_UDP_server::MSG_UDP_server(uint16 portnum)
{
  create_UDP_socket(&handle);
  set_socket_reuse(handle);
  set_socket_linger(handle);
  //  set_socket_nonblock(handle);
  bind_socket(handle,portnum);

  WARTER_add(WARTER_READ,handle);
};

MSG_UDP_server::~MSG_UDP_server()
{
  WARTER_del(WARTER_READ,handle);
  close(handle);
};

int MSG_UDP_server::getmsg(char *buf,int *size,int32 *herkunft)
{
  struct sockaddr_in from;
  int fromlen=sizeof(from);
  memset(&from,0,sizeof(from));
  int erg=recvfrom(handle, buf, *size, 0, (struct sockaddr *)&from, &fromlen);
  if (erg<0) fatalfehlererrno("MSG_UDP_server::getmsg: recvfrom(...)<0");
  *size=erg;
  *herkunft=from.sin_addr.s_addr;
  return(1);
};


*/
