/*
  
  This file is part of ERIKYYYPHONE
  Copyright (C) 1996 Erik Thiele
  see README for details and contact adresses.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  */

#include "clientconnection.H"

#include "fatal.H"
#include "msgarten.H"
#include "outconnection.H"
#include "serverconnection.H"
#include "recorder.H"
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>

liste <clientconnection> *clientconnectionlist;
liste <int32> *clientallowancelist;

void clientconnection::sendmsg(uint8 msgart,char *buf,uint16 len)
{
  liste_elem<clientconnection> *akt=clientconnectionlist->first;
  while(akt) {
    akt->objptr->com->sendmsg(msgart,buf,len); // ACHTUNG !!! wenn das element invalide ist, dann ? spaeter
    akt=akt->next;
  };
};

clientconnection::clientconnection(MSG_communicator *com_)
{
  com=com_;

  com->sendmsg(MSG_STATUS_OF_RECORDWISH,&RECORDER_moechterecord,sizeof(RECORDER_moechterecord));
  com->sendmsg(MSG_STATUS_OF_RECORD,&RECORDER_ichrecorde,sizeof(RECORDER_ichrecorde));

  {
    liste_elem<serverconnection> *akt=serverconnectionlist->first;
    while(akt) {
      akt->objptr->notifynewclient(com);
      akt=akt->next;
    };
  };
  
  {
    liste_elem<outconnection> *akt=outconnectionlist->first;
    while(akt) {
      akt->objptr->notifynewclient(com);
      akt=akt->next;
    };
  };
};

clientconnection::~clientconnection()
{
  delete com;
};

int clientconnection::handleevent()
{
  if (!(com->handleevent()))return(0);
  if (com->msgavail()) {
    char *message;
    uint8 msgart;
    uint16 len;
    com->recvmsg(&msgart,&message,&len);

    switch(msgart) 
      {
      case MSG_SERVERCONNECTION_SET:
	{
	  if (len!=sizeof(MSG_SERVERCONNECTION_SET_struct))
	    fatalfehler("clientconnection::handleevent: MSG_SERVERCONNECTION_SET: wrong len");
	  liste_elem<serverconnection> *akt=serverconnectionlist->first;
	  while(akt) {
	    if (akt->objptr->st.ip==((MSG_SERVERCONNECTION_SET_struct*)message)->ip) {
	      akt->objptr->setrellatency(((MSG_SERVERCONNECTION_SET_struct*)message)->rellatency);
	      goto dones;
	    };
	    akt=akt->next;
	  };
	  fatalfehler("clientconnection::handleevent: MSG_SERVERCONNECTION_SET: connection not found");
	dones:;
	  break;
	};
      case MSG_PLEASE_KILL_SERVERCONNECTION:
	{
	  if (len!=sizeof(int32))
	    fatalfehler("clientconnection::handleevent: MSG_PLEASE_KILL_SERVERCONNECTION: wrong len");
	  liste_elem<serverconnection> *akt=serverconnectionlist->first;
	  while(akt) {
	    if (akt->objptr->st.ip==*((int32*)message)) {
	      serverconnectionlist->del(akt);
	      goto done1;
	    };
	    akt=akt->next;
	  };
	  fatalfehler("clientconnection::handleevent: MSG_PLEASE_KILL_SERVERCONNECTION: "
		      "connection doesn't exist");
	done1:;
	  break;
	};
      case MSG_PLEASE_KILL_OUTCONNECTION:
	{
	  if (len!=sizeof(int32))
	    fatalfehler("clientconnection::handleevent: MSG_PLEASE_KILL_OUTCONNECTION: wrong len");
	  liste_elem<outconnection> *akt=outconnectionlist->first;
	  while(akt) {
	    if (akt->objptr->st.ip==*((int32*)message)) {
	      outconnectionlist->del(akt);
	      goto done2;
	    };
	    akt=akt->next;
	  };
	  fatalfehler("clientconnection::handleevent: MSG_PLEASE_KILL_OUTCONNECTION: "
		      "connection doesn't exist");
	done2:;
	  break;
	};
      case MSG_PLEASE_CONNECT_A_NEW_SERVER:
	{
	  if (len!=sizeof(int32))
	    fatalfehler("clientconnection::handleevent: MSG_PLEASE_CONNECT_A_NEW_SERVER: wrong len");

	  liste_elem<outconnection> *akt=outconnectionlist->first;
	  while(akt) {
	    if (akt->objptr->st.ip==(*((int32*)message))) {
	      char s[100],*ips;
	      MSG_ip2string(&ips,*((int32*)message));
	      sprintf(s,"I am already connected to %s",ips);
	      free(ips);
	      clientconnection::sendmsg(MSG_PLEASE_LOG_THIS_MESSAGE,s,strlen(s)+1);
	      goto fertich;
	    };
	    akt=akt->next;
	  };
	  outconnectionlist->addfront(new outconnection(*((int32*)message)));
	  WARTER_reenter=1;
	fertich:;
	  break;
	};
      case MSG_PLEASE_TOGGLE_RECORDING:
	{
	  if (len) fatalfehler("clientconnection::handleevent: MSG_PLEASE_TOGGLE_RECORDING: wrong len");
	  RECORDER_toggleaufnahme();
	  WARTER_reenter=1;
	  break;
	};
      default:
	{
	  fatalfehler("clientconnection::handleevent: wrong message received");
	};
      };
    free(message);
  };
  return(1);
};
