/*
  
  This file is part of ERIKYYYPHONE
  Copyright (C) 1996 Erik Thiele
  see README for details and contact adresses.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  */

#include "outconnection.H"

#include "fatal.H"
#include "msgarten.H"
#include "warter.H"
#include "clientconnection.H"
#include "recorder.H"
#include "serverconnection.H"

#include <stdlib.h>
#include <netinet/in.h>
#include <stdio.h>

#define MODE_init 900
#define MODE_serverready 901

liste<outconnection> *outconnectionlist;


outconnection::outconnection(int32 _ip)
{
  st.ip=_ip;
  st.status=(char)outconnection_statustype::CONNECTING;
  st.buffill=0;

  cl=new MSG_client(st.ip,htons(2900));
  commmodus=MODE_init;
  com=0;

  clientconnection::sendmsg(MSG_OUTCONNECTION_BORN,(char*)(&st),sizeof(st));
};


outconnection::~outconnection()
{
  if (com) delete com;
  if (cl) delete cl;
  clientconnection::sendmsg(MSG_OUTCONNECTION_DEAD,(char*)(&st.ip),sizeof(st.ip));
};


int outconnection::handleevent()
{
  int neuerstatus=0;
  
  if (cl) {
    int neu;
    MSG_client::statustype erg=cl->connectready(&neu);
    if (erg==MSG_client::ERROR) {
      char s[100],*ips;
      MSG_ip2string(&ips,st.ip);
      sprintf(s,"Connecting to %s failed",ips);
      free(ips);
      clientconnection::sendmsg(MSG_PLEASE_LOG_THIS_MESSAGE,s,strlen(s)+1);
      return(0);
    }else if (erg==MSG_client::CONNECTED) {
      st.status=(char)outconnection_statustype::IDLE;
      neuerstatus=1;
      delete cl;
      cl=0;
      com=new MSG_communicator(neu);
    };
  };

  if (com) {
    char *message;
    uint8 msgart;
    uint16 len;
    
    if (!(com->handleevent()))return(0);
    switch(commmodus)
      {
      case MODE_serverready:
	{
	  if (com->msgavail()) {
	    com->recvmsg(&msgart,&message,&len);
	    
	    switch(msgart)
	      {
	      case MSG_STATUS:
		{
		  if (len!=sizeof(serverconnection_statustype))
		    fatalfehler("outconnection::handleevent: MSG_STATUS: wrong length of msg");

		  st.status=((serverconnection_statustype*)message)->status;
		  st.buffill=((serverconnection_statustype*)message)->buffill;
		  neuerstatus=1;
		  break;
		};
	      };
	    free(message);
	  };
	  break;
	};
      case MODE_init:
	{
	  if (com->msgavail()) {
	    com->recvmsg(&msgart,&message,&len);
	    
	    switch(msgart)
	      {
	      case MSG_ERIKYYYPHONE_SERVERIDENTIFY_1:
		{
		  if (len!=strlen(MSG_ERIKYYYPHONE_SERVERIDENTIFY_1_BLOCK)+1)
		    fatalfehler("outconnection::handleevent: MSG_ERIKYYYPHONE_SERVERIDENTIFY_1: wrong len");
		  if (strcmp(message,MSG_ERIKYYYPHONE_SERVERIDENTIFY_1_BLOCK)) 
		    fatalfehler("outconnection::handleevent: MSG_ERIKYYYPHONE_SERVERIDENTIFY_1: wrong data");
		  commmodus=MODE_serverready;
		  com->sendmsg(MSG_ERIKYYYPHONE_IAMSERVER,NULL,0);
		  break;
		};
	      default:
		{
		  fatalfehler("outconnection::handleevent: wrong message received");
		  break;
		};
	      };
	    free(message);
	  };
	  break;
	};
      };
  };

  if (neuerstatus) clientconnection::sendmsg(MSG_OUTCONNECTION_MODIFY,(char*)(&st),sizeof(st));

  return(1);
};


void outconnection::sendeforcemsg()
{
  if (commmodus==MODE_serverready) {
    com->sendmsg(MSG_FORCE_SOUND,NULL,0);
  };
};


void outconnection::sendeblock(buffer *buf)
{
  if (commmodus==MODE_serverready) {
    com->sendmsg(MSG_V1_SOUND,buf->data,buf->bufsize);
  };
};


void outconnection::notifynewclient(MSG_communicator *x)
{
  x->sendmsg(MSG_OUTCONNECTION_BORN,(char*)(&st),sizeof(st));
};
