/*
  
  This file is part of ERIKYYYPHONE
  Copyright (C) 1996 Erik Thiele
  see README for details and contact adresses.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  */

#include "serverconnection.H"

#include "msgarten.H"
#include "fatal.H"
#include "warter.H"
#include "sound.H"
#include "clientconnection.H"

#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>

liste<serverconnection> *serverconnectionlist;

serverconnection::serverconnection(MSG_communicator *com_,int32 _ip)
{
  st.ip=_ip;
  st.status=(char)serverconnection_statustype::IDLE;
  st.buffill=0;
  st.latency=40;

  com=com_;
  buf=new buffer();
  rawbuf=new buffer();

  playgsms=0;
  erzeugeplaygsms=0;
  sleepgsms=0;
  spielaktiv=0;
  sound_aktiv=0;

  clientconnection::sendmsg(MSG_SERVERCONNECTION_BORN,(char*)(&st),sizeof(st));
};

serverconnection::~serverconnection()
{
  delete buf;
  delete com;
  delete rawbuf;
  if (sound_aktiv) SOUND_setplayingmode(0);
  if (spielaktiv) SOUND_setplaylock(0);

  clientconnection::sendmsg(MSG_SERVERCONNECTION_DEAD,(char*)(&st.ip),sizeof(st.ip));
};

int serverconnection::handleevent()
{
  int neuerstatus=0;

  if (!(com->handleevent()))return(0);
  if (com->msgavail()) {
    char *message;
    uint8 msgart;
    uint16 len;
    com->recvmsg(&msgart,&message,&len);

    switch (msgart) {
    case MSG_V1_SOUND:
      {
	if (len!=ANZ_BYTES_PRO_COMPRESSEDGSM)
	  fatalfehler("serverconnection::doit: MSG_V1_SOUND: wrong len");
	buf->changesize((buf->bufsize)+ANZ_BYTES_PRO_COMPRESSEDGSM);
	memcpy((buf->data)+(buf->bufsize)-ANZ_BYTES_PRO_COMPRESSEDGSM,message,ANZ_BYTES_PRO_COMPRESSEDGSM);
	if (erzeugeplaygsms) {
	  playgsms++;
	}else{
	  sleepgsms++;
	  if ((sleepgsms>=(int)st.latency)&&(st.latency!=-1)) {
	    playgsms+=sleepgsms;
	    sleepgsms=0;
	    erzeugeplaygsms=1;
	  };
	};
	neuerstatus=1;
	break;
      };
    case MSG_FORCE_SOUND:
      {
	if (len) fatalfehler("serverconnection::doit: MSG_POST_SOUND: wrong len");

	playgsms+=sleepgsms;
	sleepgsms=0;
	erzeugeplaygsms=0;

	neuerstatus=1;
	break;
      };
    default:
      {
	fatalfehler("serverconnection::doit: unknown message received");
      };
    };
    free(message);
  };
  
  if ((playgsms) && (!spielaktiv))
    if (SOUND_setplaylock(1)) {
      spielaktiv=1;
      neuerstatus=1;
    };
  
  if (spielaktiv) {
    while (((rawbuf->bufsize)<ANZ_SAMPLES_PRO_FRAG*2)&&(playgsms)) {
      rawbuf->changesize((rawbuf->bufsize)+ANZ_SAMPLES_PRO_GSM*2);
      SOUND_uncompressgsm((rawbuf->data)+(rawbuf->bufsize)-ANZ_SAMPLES_PRO_GSM*2,buf->data);
      buf->vornekill(ANZ_BYTES_PRO_COMPRESSEDGSM);
      playgsms--;
      neuerstatus=1;//spaeter sollte es um den buffer nicht die playgsms gehen.
    };

    if ((!erzeugeplaygsms)&&(!playgsms)&&((rawbuf->bufsize)%(ANZ_SAMPLES_PRO_FRAG*2))) {
      int fehlbytes = (ANZ_SAMPLES_PRO_FRAG*2) - ((rawbuf->bufsize) % (ANZ_SAMPLES_PRO_FRAG*2));
      rawbuf->changesize((rawbuf->bufsize)+fehlbytes);
      memset((rawbuf->data)+(rawbuf->bufsize)-fehlbytes,0,fehlbytes);
    };

    if ((rawbuf->bufsize) >= (ANZ_SAMPLES_PRO_FRAG*2)) {
      if (!sound_aktiv) {
	SOUND_setplayingmode(1);
	sound_aktiv=1;
      };
      if ((sound_aktiv)&&(SOUND_playable()))
	SOUND_playblock(rawbuf);
    }else{
      if (sound_aktiv) {
	SOUND_setplayingmode(0);
	sound_aktiv=0;
      }; 
      if (!erzeugeplaygsms) {
	if (rawbuf->bufsize)
	  fatalfehler("serverconnection::handlevent: rawbuf sollte 0 bytes gross sein !!!!! :-(");
	spielaktiv=0;
	SOUND_setplaylock(0);
	neuerstatus=1;
      };
    };
  };

  if (neuerstatus) {
    st.status=serverconnection_statustype::IDLE;
    if (sleepgsms) st.status=serverconnection_statustype::BUFFER;
    if (spielaktiv) {
      st.status=serverconnection_statustype::ON_AIR;
    }else{
      if (playgsms) st.status=serverconnection_statustype::WANT_TO_PLAY;
    };
    st.buffill=playgsms+sleepgsms;

    clientconnection::sendmsg(MSG_SERVERCONNECTION_MODIFY,(char*)(&st),sizeof(st));
    com->sendmsg(MSG_STATUS,(char*)(&st),sizeof(st));
  };

  return(1);
};

void serverconnection::notifynewclient(MSG_communicator *x)
{
  x->sendmsg(MSG_SERVERCONNECTION_BORN,(char*)(&st),sizeof(st));
};

void serverconnection::setrellatency(int rellatency)
{
  int m=st.latency;
  m+=rellatency;
  if (m<-1) {
    m=-1;
  }else if (m>80) {
    m=80;
  };
  st.latency=m;
  clientconnection::sendmsg(MSG_SERVERCONNECTION_MODIFY,(char*)(&st),sizeof(st));
};
